@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -14,11 +13,13 @@
 This module contains utility and functions to work with Green's functions calculated
 and written to ``greensf.hdf`` files by fleur
 """
+from __future__ import annotations
+
 from collections import namedtuple
 from itertools import groupby, chain
 import numpy as np
 import h5py
-from typing import List, Tuple, Iterator, Dict, Any
+from typing import Iterator, Any
 try:
     from typing import Literal
 except ImportError:
@@ -35,7 +36,7 @@ GreensfElement = namedtuple(
 CoefficientName = Literal['sphavg', 'uu', 'ud', 'du', 'dd', 'ulou', 'uulo', 'ulod', 'dulo', 'uloulo']
 
 
-def _get_sphavg_recipe(group_name: str, index: int, contour: int, version: int = None) -> Dict[str, Any]:
+def _get_sphavg_recipe(group_name: str, index: int, contour: int, version: int = None) -> dict[str, Any]:
     """
     Get the HDF5Reader recipe for reading in a spherically averaged Green's function element
 
@@ -194,7 +195,7 @@ def _get_sphavg_recipe(group_name: str, index: int, contour: int, version: int =
     return recipe
 
 
-def _get_radial_recipe(group_name: str, index: int, contour: int, nLO: int = 0, version: int = None) -> Dict[str, Any]:
+def _get_radial_recipe(group_name: str, index: int, contour: int, nLO: int = 0, version: int = None) -> dict[str, Any]:
     """
     Get the HDF5Reader recipe for reading in a radial Green's function element
 
@@ -395,7 +396,7 @@ def _get_version(hdffile: h5py.File):
     return version
 
 
-def _read_gf_element(file: Any, index: int) -> Tuple[GreensfElement, Dict[str, Any], Dict[str, Any]]:
+def _read_gf_element(file: Any, index: int) -> tuple[GreensfElement, dict[str, Any], dict[str, Any]]:
     """
     Read the information needed for a given Green's function element form a ``greensf.hdf``
     file
@@ -433,7 +434,7 @@ class GreensFunction:
     :param attributes: attributes dict produced by one of the hdf recipes for reading Green's functions
     """
 
-    def __init__(self, element: GreensfElement, data: Dict[str, Any], attributes: Dict[str, Any]) -> None:
+    def __init__(self, element: GreensfElement, data: dict[str, Any], attributes: dict[str, Any]) -> None:
         self.element = element
 
         self.points = data.pop('energy_points')
@@ -480,7 +481,7 @@ class GreensFunction:
         self.lmax = attributes['lmax']
 
     @classmethod
-    def fromFile(cls, file: Any, index: int = None, **selection_params: Any) -> 'GreensFunction':
+    def fromFile(cls, file: Any, index: int = None, **selection_params: Any) -> GreensFunction:
         """
         Classmethod for creating a :py:class:`GreensFunction` instance directly from a hdf file
 
@@ -606,7 +607,7 @@ class GreensFunction:
         return m + 3
 
     @staticmethod
-    def to_spin_indices(spin: int) -> Tuple[int, int]:
+    def to_spin_indices(spin: int) -> tuple[int, int]:
         """
         Convert between spin index (0 to 3) to the corresponding
         two spin indices (0 or 1)
@@ -730,7 +731,7 @@ class colors:
     green = '\033[32m'
 
 
-def printElements(elements: List[GreensfElement], index: List[int] = None, mark: List[int] = None) -> None:
+def printElements(elements: list[GreensfElement], index: list[int] = None, mark: list[int] = None) -> None:
     """
     Print the given list of :py:class:`GreensfElement` in a nice table
 
@@ -765,7 +766,7 @@ def printElements(elements: List[GreensfElement], index: List[int] = None, mark:
             + colors.endc)
 
 
-def listElements(hdffile: Any, show: bool = False) -> List[GreensfElement]:
+def listElements(hdffile: Any, show: bool = False) -> list[GreensfElement]:
     """
     Find the green's function elements contained in the given ``greens.hdf`` file
 
@@ -813,7 +814,7 @@ def select_elements_from_file(hdffile: Any, show: bool = False, **selection_para
     return gf_iterator(found_elements)
 
 
-def select_elements(greensfunctions: List[GreensFunction],
+def select_elements(greensfunctions: list[GreensFunction],
                     show: bool = False,
                     **selection_params) -> Iterator[GreensFunction]:
     """
@@ -837,7 +838,7 @@ def select_elements(greensfunctions: List[GreensFunction],
     return gf_iterator(found_elements)
 
 
-def select_element_indices(elements: List[GreensfElement], show: bool = False, **selection_params) -> List[int]:
+def select_element_indices(elements: list[GreensfElement], show: bool = False, **selection_params) -> list[int]:
     """
     Select :py:class:`GreensfElement` objects from a list based on constraints on their
     values
@@ -868,7 +869,7 @@ def select_element_indices(elements: List[GreensfElement], show: bool = False, *
 
 def intersite_shells_from_file(hdffile: Any,
                                reference_atom: int,
-                               show: bool = False) -> Iterator[Tuple[float, GreensFunction, GreensFunction]]:
+                               show: bool = False) -> Iterator[tuple[float, GreensFunction, GreensFunction]]:
     """
     Construct the green's function pairs to calculate the Jij exchange constants
     for a given reference atom from a given ``greensf.hdf`` file
@@ -895,9 +896,9 @@ def intersite_shells_from_file(hdffile: Any,
     return shell_iterator(jij_pairs)
 
 
-def intersite_shells(greensfunctions: List[GreensFunction],
+def intersite_shells(greensfunctions: list[GreensFunction],
                      reference_atom: int,
-                     show: bool = False) -> Iterator[Tuple[float, GreensFunction, GreensFunction]]:
+                     show: bool = False) -> Iterator[tuple[float, GreensFunction, GreensFunction]]:
     """
     Construct the green's function pairs to calculate the Jij exchange constants
     for a given reference atom from a list of given :py:class:`GreensFunction`
@@ -921,9 +922,9 @@ def intersite_shells(greensfunctions: List[GreensFunction],
     return shell_iterator(jij_pairs)
 
 
-def intersite_shell_indices(elements: List[GreensfElement],
+def intersite_shell_indices(elements: list[GreensfElement],
                             reference_atom: int,
-                            show: bool = False) -> List[Tuple[float, List[Tuple[int, int]]]]:
+                            show: bool = False) -> list[tuple[float, list[tuple[int, int]]]]:
     """
     Construct the green's function pairs to calculate the Jij exchange constants
     for a given reference atom from a list of :py:class:`GreensfElement`
