@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -17,14 +16,24 @@ schema_dicts defined for the Fleur input/output
 Also provides convienient functions to use just a attribute name for extracting the
 attribute from the right place in the given etree
 """
+from __future__ import annotations
+
 from masci_tools.io.parsers.fleur_schema import NoPathFound
 from masci_tools.util.parse_tasks_decorators import register_parsing_function
 from masci_tools.io.parsers import fleur_schema
-from masci_tools.util.xml.common_functions import check_complex_xpath
+from masci_tools.util.xml.common_functions import add_tag, check_complex_xpath
+from masci_tools.util.xml.xpathbuilder import XPathBuilder, FilterType
+from masci_tools.util.typing import XPathLike, XMLLike
 from lxml import etree
 from logging import Logger
 import warnings
-from typing import Dict, Iterable, Tuple, Union, Any, List
+import copy
+import os
+from typing import Iterable, Any, overload
+try:
+    from typing import Literal
+except ImportError:
+    from typing_extensions import Literal  #type:ignore
 
 
 def get_tag_xpath(schema_dict, name, contains=None, not_contains=None):
@@ -176,9 +185,9 @@ def get_tag_info(schema_dict,
                                 parent=parent)
 
 
-def read_constants(root: Union[etree._Element, etree._ElementTree],
-                   schema_dict: 'fleur_schema.SchemaDict',
-                   logger: Logger = None) -> Dict[str, float]:
+def read_constants(root: XMLLike,
+                   schema_dict: fleur_schema.SchemaDict,
+                   logger: Logger | None = None) -> dict[str, float]:
     """
     Reads in the constants defined in the inp.xml
     and returns them combined with the predefined constants from
@@ -191,7 +200,6 @@ def read_constants(root: Union[etree._Element, etree._ElementTree],
     :return: a python dictionary with all defined constants
     """
     from masci_tools.util.constants import FLEUR_DEFINED_CONSTANTS
-    import copy
 
     defined_constants = copy.deepcopy(FLEUR_DEFINED_CONSTANTS)
 
@@ -221,12 +229,13 @@ def read_constants(root: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('attrib')
-def evaluate_attribute(node: Union[etree._Element, etree._ElementTree],
-                       schema_dict: 'fleur_schema.SchemaDict',
+def evaluate_attribute(node: XMLLike,
+                       schema_dict: fleur_schema.SchemaDict,
                        name: str,
-                       constants: Dict[str, float] = None,
-                       logger: Logger = None,
-                       complex_xpath: 'etree._xpath' = None,
+                       constants: dict[str, float] | None = None,
+                       logger: Logger | None = None,
+                       complex_xpath: XPathLike | None = None,
+                       filters: FilterType | None = None,
                        iteration_path: bool = False,
                        **kwargs: Any) -> Any:
     """
@@ -241,6 +250,8 @@ def evaluate_attribute(node: Union[etree._Element, etree._ElementTree],
     :param complex_xpath: an optional xpath to use instead of the simple xpath for the evaluation
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     Kwargs:
         :param tag_name: str, name of the tag where the attribute should be parsed
@@ -262,11 +273,16 @@ def evaluate_attribute(node: Union[etree._Element, etree._ElementTree],
     attrib_xpath = _select_attrib_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
 
     if complex_xpath is None:
-        complex_xpath = attrib_xpath
-
+        complex_xpath = XPathBuilder(attrib_xpath, filters=filters, strict=True)
+    elif filters is not None:
+        if not isinstance(complex_xpath, XPathBuilder):
+            raise ValueError(
+                'Provide only one of filters or complex_xpath (Except when complx_xpath is given as a XPathBuilder)')
+        for key, val in filters.items():
+            complex_xpath.add_filter(key, val)
     check_complex_xpath(node, attrib_xpath, complex_xpath)
 
-    stringattribute: List[str] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
+    stringattribute: list[str] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
 
     if len(stringattribute) == 0:
         if logger is None:
@@ -295,13 +311,14 @@ def evaluate_attribute(node: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('text')
-def evaluate_text(node: Union[etree._Element, etree._ElementTree],
-                  schema_dict: 'fleur_schema.SchemaDict',
+def evaluate_text(node: XMLLike,
+                  schema_dict: fleur_schema.SchemaDict,
                   name: str,
-                  constants: Dict[str, float] = None,
-                  logger: Logger = None,
-                  complex_xpath: 'etree._xpath' = None,
+                  constants: dict[str, float] | None = None,
+                  logger: Logger | None = None,
+                  complex_xpath: XPathLike | None = None,
                   iteration_path: bool = False,
+                  filters: FilterType | None = None,
                   **kwargs: Any) -> Any:
     """
     Evaluates the text of the tag based on the given name
@@ -315,6 +332,8 @@ def evaluate_text(node: Union[etree._Element, etree._ElementTree],
     :param complex_xpath: an optional xpath to use instead of the simple xpath for the evaluation
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     Kwargs:
         :param contains: str, this string has to be in the final path
@@ -332,11 +351,17 @@ def evaluate_text(node: Union[etree._Element, etree._ElementTree],
 
     tag_xpath = _select_tag_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
     if complex_xpath is None:
-        complex_xpath = tag_xpath
+        complex_xpath = XPathBuilder(tag_xpath, filters=filters, strict=True)
+    elif filters is not None:
+        if not isinstance(complex_xpath, XPathBuilder):
+            raise ValueError(
+                'Provide only one of filters or complex_xpath (Except when complx_xpath is given as a XPathBuilder)')
+        for key, val in filters.items():
+            complex_xpath.add_filter(key, val)
 
     check_complex_xpath(node, tag_xpath, complex_xpath)
 
-    stringtext: List[str] = eval_xpath(node, f'{str(complex_xpath)}/text()', logger=logger,
+    stringtext: list[str] = eval_xpath(node, add_tag(complex_xpath, 'text()'), logger=logger,
                                        list_return=True)  #type:ignore
 
     for text in stringtext.copy():
@@ -370,15 +395,16 @@ def evaluate_text(node: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('allAttribs', all_attribs_keys=True)
-def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
-                 schema_dict: 'fleur_schema.SchemaDict',
+def evaluate_tag(node: XMLLike,
+                 schema_dict: fleur_schema.SchemaDict,
                  name: str,
-                 constants: Dict[str, float] = None,
-                 logger: Logger = None,
+                 constants: dict[str, float] | None = None,
+                 logger: Logger | None = None,
                  subtags: bool = False,
                  text: bool = True,
-                 complex_xpath: 'etree._xpath' = None,
+                 complex_xpath: XPathLike | None = None,
                  iteration_path: bool = False,
+                 filters: FilterType | None = None,
                  **kwargs: Any) -> Any:
     """
     Evaluates all attributes of the tag based on the given name
@@ -394,6 +420,8 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
     :param complex_xpath: an optional xpath to use instead of the simple xpath for the evaluation
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     Kwargs:
         :param contains: str, this string has to be in the final path
@@ -415,7 +443,13 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
 
     tag_xpath = _select_tag_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
     if complex_xpath is None:
-        complex_xpath = tag_xpath
+        complex_xpath = XPathBuilder(tag_xpath, filters=filters, strict=True)
+    elif filters is not None:
+        if not isinstance(complex_xpath, XPathBuilder):
+            raise ValueError(
+                'Provide only one of filters or complex_xpath (Except when complx_xpath is given as a XPathBuilder)')
+        for key, val in filters.items():
+            complex_xpath.add_filter(key, val)
 
     check_complex_xpath(node, tag_xpath, complex_xpath)
 
@@ -460,12 +494,12 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
             'exist or it has no attributes', name)
     attrib_list = sorted(list(attribs.original_case.values()))
 
-    out_dict: Dict[str, Any] = {}
+    out_dict: dict[str, Any] = {}
 
     for attrib in attrib_list:
 
-        stringattribute: List[str] = eval_xpath(node,
-                                                f'{str(complex_xpath)}/@{attrib}',
+        stringattribute: list[str] = eval_xpath(node,
+                                                add_tag(complex_xpath, f'@{attrib}'),
                                                 logger=logger,
                                                 list_return=True)  #type:ignore
 
@@ -496,7 +530,7 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
     if parse_text:
 
         _, name = split_off_tag(tag_xpath)
-        stringtext: List[str] = eval_xpath(node, f'{str(complex_xpath)}/text()', logger=logger,
+        stringtext: list[str] = eval_xpath(node, add_tag(complex_xpath, 'text()'), logger=logger,
                                            list_return=True)  #type:ignore
 
         for textval in stringtext.copy():
@@ -526,11 +560,11 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
         for tag in tags:
             if tag in out_dict:
                 if logger is None:
-                    raise ValueError(f'Conflicting key {tag}: ' 'Key is already in the output dictionary')
-                logger.error('Conflicting key %s: ' 'Key is already in the output dictionary', tag)
+                    raise ValueError(f'Conflicting key {tag}: Key is already in the output dictionary')
+                logger.error('Conflicting key %s: Key is already in the output dictionary', tag)
             out_dict[tag] = []
 
-        sub_nodes: List[etree._Element] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
+        sub_nodes: list[etree._Element] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
         for sub_node in sub_nodes:
             for tag in tags:
                 if tag_exists(sub_node, schema_dict, tag):
@@ -564,12 +598,12 @@ def evaluate_tag(node: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('singleValue', all_attribs_keys=True)
-def evaluate_single_value_tag(node: Union[etree._Element, etree._ElementTree],
-                              schema_dict: 'fleur_schema.SchemaDict',
+def evaluate_single_value_tag(node: XMLLike,
+                              schema_dict: fleur_schema.SchemaDict,
                               name: str,
-                              constants: Dict[str, float] = None,
-                              logger: Logger = None,
-                              complex_xpath: 'etree._xpath' = None,
+                              constants: dict[str, float] | None = None,
+                              logger: Logger | None = None,
+                              complex_xpath: XPathLike | None = None,
                               **kwargs: Any) -> Any:
     """
     Evaluates the value and unit attribute of the tag based on the given name
@@ -591,12 +625,14 @@ def evaluate_single_value_tag(node: Union[etree._Element, etree._ElementTree],
         :param strict_missing_error: if True, and no logger is given an error is raised if any attribute is not found
         :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                                the iteration element is constructed
+        :param filters: Dict specifying constraints to apply on the xpath.
+                        See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     :returns: value and unit, both converted in convert_xml_attribute
     """
 
     only_required = kwargs.get('only_required', False)
-    ignore = kwargs.get('ignore', [])
+    ignore = kwargs.pop('ignore', ['comment'])
 
     value_dict = evaluate_tag(node,
                               schema_dict,
@@ -604,29 +640,31 @@ def evaluate_single_value_tag(node: Union[etree._Element, etree._ElementTree],
                               constants=constants,
                               logger=logger,
                               complex_xpath=complex_xpath,
+                              ignore=ignore,
                               **kwargs)
 
     if value_dict.get('value') is None:
         if logger is None:
-            raise ValueError(f'Failed to evaluate singleValue from tag {name}: ' "Has no 'value' attribute")
-        logger.warning('Failed to evaluate singleValue from tag %s: ' "Has no 'value' attribute", name)
+            raise ValueError(f"Failed to evaluate singleValue from tag {name}: Has no 'value' attribute")
+        logger.warning("Failed to evaluate singleValue from tag %s: Has no 'value' attribute", name)
 
     if value_dict.get('units') is None and not only_required and 'units' not in ignore:
         if logger is None:
-            raise ValueError(f'Failed to evaluate singleValue from tag {name}: ' "Has no 'units' attribute")
-        logger.warning('Failed to evaluate singleValue from tag %s: ' "Has no 'units' attribute", name)
+            raise ValueError(f"Failed to evaluate singleValue from tag {name}: Has no 'units' attribute")
+        logger.warning("Failed to evaluate singleValue from tag %s: Has no 'units' attribute", name)
 
     return value_dict
 
 
 @register_parsing_function('parentAttribs', all_attribs_keys=True)
-def evaluate_parent_tag(node: Union[etree._Element, etree._ElementTree],
-                        schema_dict: 'fleur_schema.SchemaDict',
+def evaluate_parent_tag(node: XMLLike,
+                        schema_dict: fleur_schema.SchemaDict,
                         name: str,
-                        constants: Dict[str, float] = None,
-                        logger: Logger = None,
-                        complex_xpath: 'etree._xpath' = None,
+                        constants: dict[str, float] | None = None,
+                        logger: Logger | None = None,
+                        complex_xpath: XPathLike | None = None,
                         iteration_path: bool = False,
+                        filters: FilterType | None = None,
                         **kwargs: Any) -> Any:
     """
     Evaluates all attributes of the parent tag based on the given name
@@ -640,6 +678,8 @@ def evaluate_parent_tag(node: Union[etree._Element, etree._ElementTree],
     :param complex_xpath: an optional xpath to use instead of the simple xpath for the evaluation
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     Kwargs:
         :param contains: str, this string has to be in the final path
@@ -661,7 +701,13 @@ def evaluate_parent_tag(node: Union[etree._Element, etree._ElementTree],
 
     tag_xpath = _select_tag_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
     if complex_xpath is None:
-        complex_xpath = tag_xpath
+        complex_xpath = XPathBuilder(tag_xpath, filters=filters, strict=True)
+    elif filters is not None:
+        if not isinstance(complex_xpath, XPathBuilder):
+            raise ValueError(
+                'Provide only one of filters or complex_xpath (Except when complx_xpath is given as a XPathBuilder)')
+        for key, val in filters.items():
+            complex_xpath.add_filter(key, val)
 
     check_complex_xpath(node, tag_xpath, complex_xpath)
 
@@ -699,9 +745,9 @@ def evaluate_parent_tag(node: Union[etree._Element, etree._ElementTree],
             'exist or it has no attributes', name)
     attrib_list = sorted(list(attribs.original_case.values()))
 
-    elems: List[etree._Element] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
+    elems: list[etree._Element] = eval_xpath(node, complex_xpath, logger=logger, list_return=True)  #type:ignore
 
-    out_dict: Dict[str, Any] = {}
+    out_dict: dict[str, Any] = {}
     for attrib in attrib_list:
         out_dict[attrib] = []
 
@@ -750,11 +796,12 @@ def evaluate_parent_tag(node: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('attrib_exists')
-def attrib_exists(node: Union[etree._Element, etree._ElementTree],
-                  schema_dict: 'fleur_schema.SchemaDict',
+def attrib_exists(node: XMLLike,
+                  schema_dict: fleur_schema.SchemaDict,
                   name: str,
-                  logger: Logger = None,
+                  logger: Logger | None = None,
                   iteration_path: bool = False,
+                  filters: FilterType | None = None,
                   **kwargs: Any) -> bool:
     """
     Evaluates whether the attribute exists in the xmltree based on the given name
@@ -766,6 +813,8 @@ def attrib_exists(node: Union[etree._Element, etree._ElementTree],
     :param logger: logger object for logging warnings, errors, if not provided all errors will be raised
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     Kwargs:
         :param tag_name: str, name of the tag where the attribute should be parsed
@@ -780,17 +829,17 @@ def attrib_exists(node: Union[etree._Element, etree._ElementTree],
 
     attrib_xpath = _select_attrib_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
     tag_xpath, attrib_name = split_off_attrib(attrib_xpath)
+    tag_xpath_builder = XPathBuilder(tag_xpath, filters=filters, strict=True)
 
-    tags: List[etree._Element] = eval_xpath(node, tag_xpath, logger=logger, list_return=True)  #type:ignore
-
+    tags: list[etree._Element] = eval_xpath(node, tag_xpath_builder, logger=logger, list_return=True)  #type:ignore
     return any(attrib_name in tag.attrib for tag in tags)
 
 
 @register_parsing_function('exists')
-def tag_exists(node: Union[etree._Element, etree._ElementTree],
-               schema_dict: 'fleur_schema.SchemaDict',
+def tag_exists(node: XMLLike,
+               schema_dict: fleur_schema.SchemaDict,
                name: str,
-               logger: Logger = None,
+               logger: Logger | None = None,
                **kwargs: Any) -> bool:
     """
     Evaluates whether the tag exists in the xmltree based on the given name
@@ -806,6 +855,8 @@ def tag_exists(node: Union[etree._Element, etree._ElementTree],
         :param not_contains: str, this string has to NOT be in the final path
         :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+        :param filters: Dict specifying constraints to apply on the xpath.
+                        See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     :returns: bool, True if any nodes with the path exist
     """
@@ -813,10 +864,10 @@ def tag_exists(node: Union[etree._Element, etree._ElementTree],
 
 
 @register_parsing_function('numberNodes')
-def get_number_of_nodes(node: Union[etree._Element, etree._ElementTree],
-                        schema_dict: 'fleur_schema.SchemaDict',
+def get_number_of_nodes(node: XMLLike,
+                        schema_dict: fleur_schema.SchemaDict,
                         name: str,
-                        logger: Logger = None,
+                        logger: Logger | None = None,
                         **kwargs: Any) -> int:
     """
     Evaluates the number of occurences of the tag in the xmltree based on the given name
@@ -832,6 +883,8 @@ def get_number_of_nodes(node: Union[etree._Element, etree._ElementTree],
         :param not_contains: str, this string has to NOT be in the final path
         :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+        :param filters: Dict specifying constraints to apply on the xpath.
+                        See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
 
     :returns: bool, True if any nodes with the path exist
     """
@@ -841,12 +894,38 @@ def get_number_of_nodes(node: Union[etree._Element, etree._ElementTree],
     return len(result)
 
 
-def eval_simple_xpath(node: Union[etree._Element, etree._ElementTree],
-                      schema_dict: 'fleur_schema.SchemaDict',
+@overload
+def eval_simple_xpath(node: XMLLike,
+                      schema_dict: fleur_schema.SchemaDict,
                       name: str,
-                      logger: Logger = None,
+                      logger: Logger | None = None,
                       iteration_path: bool = False,
-                      **kwargs: Any) -> 'etree._XPathObject':
+                      filters: FilterType | None = None,
+                      list_return: Literal[True] = ...,
+                      **kwargs: Any) -> list[etree._Element]:
+    ...
+
+
+@overload
+def eval_simple_xpath(node: XMLLike,
+                      schema_dict: fleur_schema.SchemaDict,
+                      name: str,
+                      logger: Logger | None = None,
+                      iteration_path: bool = False,
+                      filters: FilterType | None = None,
+                      list_return: Literal[False] = ...,
+                      **kwargs: Any) -> etree._Element | list[etree._Element]:
+    ...
+
+
+def eval_simple_xpath(node: XMLLike,
+                      schema_dict: fleur_schema.SchemaDict,
+                      name: str,
+                      logger: Logger | None = None,
+                      iteration_path: bool = False,
+                      filters: FilterType | None = None,
+                      list_return: bool = False,
+                      **kwargs: Any) -> etree._Element | list[etree._Element]:
     """
     Evaluates a simple xpath expression of the tag in the xmltree based on the given name
     and additional further specifications with the available type information
@@ -857,23 +936,121 @@ def eval_simple_xpath(node: Union[etree._Element, etree._ElementTree],
     :param logger: logger object for logging warnings, errors, if not provided all errors will be raised
     :param iteration_path: bool if True and the SchemaDict is of an output schema an absolute path into
                            the iteration element is constructed
+    :param filters: Dict specifying constraints to apply on the xpath.
+                    See :py:class:`~masci_tools.util.xml.xpathbuilder.XPathBuilder` for details
+    :param list_return: bool, if True a list is always returned
 
     Kwargs:
         :param contains: str, this string has to be in the final path
         :param not_contains: str, this string has to NOT be in the final path
-        :param list_return: bool, if True a list is always returned
 
     :returns: etree Elements obtained via the simple xpath expression
     """
     from masci_tools.util.xml.common_functions import eval_xpath
 
-    list_return = kwargs.pop('list_return', False)
     tag_xpath = _select_tag_xpath(node, schema_dict, name, iteration_path=iteration_path, **kwargs)
-    return eval_xpath(node, tag_xpath, logger=logger, list_return=list_return)
+    tag_xpath_builder = XPathBuilder(tag_xpath, strict=True, filters=filters)
+
+    return eval_xpath(node, tag_xpath_builder, logger=logger, list_return=list_return)  #type: ignore[return-value]
 
 
-def _select_tag_xpath(node: Union[etree._Element, etree._ElementTree],
-                      schema_dict: 'fleur_schema.SchemaDict',
+def reverse_xinclude(xmltree: etree._ElementTree, schema_dict: fleur_schema.SchemaDict, included_tags: Iterable[str],
+                     **kwargs: os.PathLike) -> tuple[etree._ElementTree, dict[os.PathLike | str, etree._ElementTree]]:
+    """
+    Split the xmltree back up according to the given included tags.
+    The original xmltree will be returned with the corresponding xinclude tags
+    and the included trees are returned in a dict mapping the inserted filename
+    to the extracted tree
+
+    Tags for which no known filename is known are returned under unknown-1.xml, ...
+    The following tags have known filenames:
+
+        - `relaxation`: ``relax.xml``
+        - `kPointLists`: ``kpts.xml``
+        - `symmetryOperations`: ``sym.xml``
+        - `atomSpecies`: ``species.xml``
+        - `atomGroups`: ``atoms.xml``
+
+    Additional mappings can be given in the keyword arguments
+
+    :param xmltree: an xml-tree which will be processed
+    :param schema_dict: Schema dictionary containing all the necessary information
+    :param included_tags: Iterable of str, containing the names of the tags to be excluded
+
+    :returns: xmltree with the inseerted xinclude tags and a dict mapping the filenames
+              to the excluded trees
+
+    :raises ValueError: if the tag can not be found in teh given xmltree
+    """
+    from masci_tools.util.xml.common_functions import eval_xpath
+
+    INCLUDE_NSMAP = {'xi': 'http://www.w3.org/2001/XInclude'}
+    INCLUDE_TAG = etree.QName(INCLUDE_NSMAP['xi'], 'include')
+    FALLBACK_TAG = etree.QName(INCLUDE_NSMAP['xi'], 'fallback')
+
+    excluded_tree = copy.deepcopy(xmltree)
+
+    include_file_names: dict[str, os.PathLike | str] = {
+        'relaxation': 'relax.xml',
+        'kPointLists': 'kpts.xml',
+        'symmetryOperations': 'sym.xml',
+        'atomSpecies': 'species.xml',
+        'atomGroups': 'atoms.xml'
+    }
+
+    include_file_names = {**include_file_names, **kwargs}
+
+    unknown_file_names = 0
+    included_trees = {}
+    root = excluded_tree.getroot()
+
+    if not all(isinstance(tag, str) for tag in included_tags):
+        raise ValueError(f'included_tags is not made up of strings: {included_tags}')
+
+    for tag in included_tags:
+        if tag in include_file_names:
+            file_name = include_file_names[tag]
+        else:
+            warnings.warn(f'No filename known for tag {tag}')
+            unknown_file_names += 1
+            file_name = f'unknown-{unknown_file_names}.xml'
+
+        try:
+            tag_xpath = schema_dict.tag_xpath(tag)
+        except Exception as err:
+            raise ValueError(f'Cannot determine place of included tag {tag}') from err
+        included_tag_res: list[etree._Element] = eval_xpath(root, tag_xpath, list_return=True)  #type:ignore
+
+        if len(included_tag_res) != 1:
+            raise ValueError(f'Cannot determine place of included tag {tag}')
+        included_tag = included_tag_res[0]
+
+        included_trees[file_name] = etree.ElementTree(included_tag)
+
+        parent = included_tag.getparent()
+        if parent is None:
+            raise ValueError('Could not find parent of included tag')
+
+        xinclude_elem = etree.Element(INCLUDE_TAG, href=os.fspath(file_name), nsmap=INCLUDE_NSMAP)  #type:ignore
+        xinclude_elem.append(etree.Element(FALLBACK_TAG))  #type:ignore
+
+        parent.replace(included_tag, xinclude_elem)
+
+    if 'relax.xml' not in included_trees:
+        #The relax.xml include should always be there
+        xinclude_elem = etree.Element(INCLUDE_TAG, href='relax.xml', nsmap=INCLUDE_NSMAP)  #type:ignore
+        xinclude_elem.append(etree.Element(FALLBACK_TAG))  #type:ignore
+        root.append(xinclude_elem)
+
+    etree.indent(excluded_tree)
+    for tree in included_trees.values():
+        etree.indent(tree)
+
+    return excluded_tree, included_trees
+
+
+def _select_tag_xpath(node: XMLLike,
+                      schema_dict: fleur_schema.SchemaDict,
                       name: str,
                       iteration_path: bool = False,
                       **kwargs: Any) -> str:
@@ -905,7 +1082,7 @@ def _select_tag_xpath(node: Union[etree._Element, etree._ElementTree],
         if not isinstance(schema_dict, fleur_schema.OutputSchemaDict):
             raise ValueError('iteration_path=True can only be used with OutputSchemaDict')
 
-    root_tags: Tuple[str, ...] = (schema_dict['root_tag'],)
+    root_tags: tuple[str, ...] = (schema_dict['root_tag'],)
     if isinstance(schema_dict, fleur_schema.OutputSchemaDict):
         root_tags += tuple(schema_dict['iteration_tags'])
 
@@ -926,8 +1103,8 @@ def _select_tag_xpath(node: Union[etree._Element, etree._ElementTree],
     return xpath
 
 
-def _select_attrib_xpath(node: Union[etree._Element, etree._ElementTree],
-                         schema_dict: 'fleur_schema.SchemaDict',
+def _select_attrib_xpath(node: XMLLike,
+                         schema_dict: fleur_schema.SchemaDict,
                          name: str,
                          iteration_path: bool = False,
                          **kwargs: Any) -> str:
@@ -962,7 +1139,7 @@ def _select_attrib_xpath(node: Union[etree._Element, etree._ElementTree],
         if not isinstance(schema_dict, fleur_schema.OutputSchemaDict):
             raise ValueError('iteration_path=True can only be used with OutputSchemaDict')
 
-    root_tags: Tuple[str, ...] = (schema_dict['root_tag'],)
+    root_tags: tuple[str, ...] = (schema_dict['root_tag'],)
     if isinstance(schema_dict, fleur_schema.OutputSchemaDict):
         root_tags += tuple(schema_dict['iteration_tags'])
 
@@ -983,12 +1160,12 @@ def _select_attrib_xpath(node: Union[etree._Element, etree._ElementTree],
     return xpath
 
 
-def _select_tag_info(node: Union[etree._Element, etree._ElementTree],
-                     schema_dict: 'fleur_schema.SchemaDict',
+def _select_tag_info(node: XMLLike,
+                     schema_dict: fleur_schema.SchemaDict,
                      name: str,
                      iteration_path: bool = False,
                      iteration_tag: str = 'iteration',
-                     contains: Union[str, Iterable[str]] = None,
+                     contains: str | Iterable[str] | None = None,
                      **kwargs: Any) -> fleur_schema.schema_dict.TagInfo:
     """
     Get the tag information used for the evaluation function in this module
@@ -1021,7 +1198,7 @@ def _select_tag_info(node: Union[etree._Element, etree._ElementTree],
         if not isinstance(schema_dict, fleur_schema.OutputSchemaDict):
             raise ValueError('iteration_path=True can only be used with OutputSchemaDict')
 
-    root_tags: Tuple[str, ...] = (schema_dict['root_tag'],)
+    root_tags: tuple[str, ...] = (schema_dict['root_tag'],)
     if isinstance(schema_dict, fleur_schema.OutputSchemaDict):
         root_tags += tuple(schema_dict['iteration_tags'])
 
